@@ -3,10 +3,11 @@
 
 import io
 import os
+from collections.abc import Mapping
 from enum import Enum, IntEnum
 from gettext import gettext as _
 from tempfile import TemporaryFile
-from typing import IO, Callable, Dict, List, Mapping, NamedTuple, Optional, Tuple, Union
+from typing import IO, Callable, NamedTuple, Optional, Union
 
 from .conf.utils import uniq
 from .constants import supports_primary_selection
@@ -14,6 +15,7 @@ from .fast_data_types import (
     ESC_OSC,
     GLFW_CLIPBOARD,
     GLFW_PRIMARY_SELECTION,
+    StreamingBase64Decoder,
     find_in_memoryview,
     get_boss,
     get_clipboard_mime,
@@ -82,7 +84,7 @@ class ClipboardType(IntEnum):
 class Clipboard:
 
     def __init__(self, clipboard_type: ClipboardType = ClipboardType.clipboard) -> None:
-        self.data: Dict[str, DataType] = {}
+        self.data: dict[str, DataType] = {}
         self.clipboard_type = clipboard_type
         self.enabled = self.clipboard_type is ClipboardType.clipboard or supports_primary_selection
 
@@ -97,7 +99,7 @@ class Clipboard:
             set_clipboard_data_types(self.clipboard_type, tuple(self.data))
 
     def get_text(self) -> str:
-        parts: List[bytes] = []
+        parts: list[bytes] = []
         self.get_mime("text/plain", parts.append)
         return b''.join(parts).decode('utf-8', 'replace')
 
@@ -119,13 +121,13 @@ class Clipboard:
                         output(q)
 
     def get_mime_data(self, mime: str) -> bytes:
-        ans: List[bytes] = []
+        ans: list[bytes] = []
         self.get_mime(mime, ans.append)
         return b''.join(ans)
 
-    def get_available_mime_types_for_paste(self) -> Tuple[str, ...]:
+    def get_available_mime_types_for_paste(self) -> tuple[str, ...]:
         if self.enabled:
-            parts: List[bytes] = []
+            parts: list[bytes] = []
             try:
                 get_clipboard_mime(self.clipboard_type, None, parts.append)
             except RuntimeError as err:
@@ -167,7 +169,7 @@ def get_primary_selection() -> str:
     return get_boss().primary_selection.get_text()
 
 
-def develop() -> Tuple[Clipboard, Clipboard]:
+def develop() -> tuple[Clipboard, Clipboard]:
     from .constants import detect_if_wayland_ok, is_macos
     from .fast_data_types import set_boss
     from .main import init_glfw_module
@@ -200,7 +202,7 @@ def decode_metadata_value(k: str, x: str) -> str:
 
 class ReadRequest(NamedTuple):
     is_primary_selection: bool = False
-    mime_types: Tuple[str, ...] = ('text/plain',)
+    mime_types: tuple[str, ...] = ('text/plain',)
     id: str = ''
     protocol_type: ProtocolType = ProtocolType.osc_52
 
@@ -236,16 +238,16 @@ class WriteRequest:
         self, is_primary_selection: bool = False, protocol_type: ProtocolType = ProtocolType.osc_52, id: str = '',
         rollover_size: int = 16 * 1024 * 1024, max_size: int = -1,
     ) -> None:
+        self.decoder = StreamingBase64Decoder()
         self.id = id
         self.is_primary_selection = is_primary_selection
         self.protocol_type = protocol_type
         self.max_size_exceeded = False
         self.tempfile = Tempfile(max_size=rollover_size)
-        self.mime_map: Dict[str, MimePos] = {}
+        self.mime_map: dict[str, MimePos] = {}
         self.currently_writing_mime = ''
-        self.current_leftover_bytes = memoryview(b'')
         self.max_size = (get_options().clipboard_max_size * 1024 * 1024) if max_size < 0 else max_size
-        self.aliases: Dict[str, str] = {}
+        self.aliases: dict[str, str] = {}
         self.committed = False
 
     def encode_response(self, status: str = 'OK') -> bytes:
@@ -276,51 +278,28 @@ class WriteRequest:
         if not self.currently_writing_mime:
             self.mime_map[mime] = MimePos(self.tempfile.tell(), -1)
             self.currently_writing_mime = mime
-
-        def write_saving_leftover_bytes(data: bytes) -> None:
-            if len(data) == 0:
-                return
-            extra = len(data) % 4
-            if extra > 0:
-                mv = memoryview(data)
-                self.current_leftover_bytes = memoryview(bytes(mv[-extra:]))
-                mv = mv[:-extra]
-                if len(mv) > 0:
-                    self.write_base64_data(mv)
-            else:
-                self.write_base64_data(data)
-
-        if len(self.current_leftover_bytes) > 0:
-            extra = 4 - len(self.current_leftover_bytes)
-            if len(data) >= extra:
-                self.write_base64_data(memoryview(bytes(self.current_leftover_bytes) + data[:extra]))
-                self.current_leftover_bytes = memoryview(b'')
-                data = memoryview(data)[extra:]
-                write_saving_leftover_bytes(data)
-            else:
-                self.current_leftover_bytes = memoryview(bytes(self.current_leftover_bytes) + data)
-        else:
-            write_saving_leftover_bytes(data)
+        self.write_base64_data(data)
 
     def flush_base64_data(self) -> None:
         if self.currently_writing_mime:
-            b = self.current_leftover_bytes
-            padding = 4 - len(b)
-            if padding in (1, 2):
-                self.write_base64_data(memoryview(bytes(b) + b'=' * padding))
+            self.decoder.reset()
             start = self.mime_map[self.currently_writing_mime][0]
             self.mime_map[self.currently_writing_mime] = MimePos(start, self.tempfile.tell() - start)
             self.currently_writing_mime = ''
-            self.current_leftover_bytes = memoryview(b'')
 
     def write_base64_data(self, b: bytes) -> None:
-        from base64 import standard_b64decode
         if not self.max_size_exceeded:
-            d = standard_b64decode(b)
-            self.tempfile.write(d)
-            if self.max_size > 0 and self.tempfile.tell() > (self.max_size * 1024 * 1024):
-                log_error(f'Clipboard write request has more data than allowed by clipboard_max_size ({self.max_size}), truncating')
-                self.max_size_exceeded = True
+            try:
+                decoded = self.decoder.decode(b)
+            except ValueError:
+                log_error('Clipboard write request has invalid data, ignoring this chunk of data')
+                self.decoder.reset()
+                decoded = b''
+            if decoded:
+                self.tempfile.write(decoded)
+                if self.max_size > 0 and self.tempfile.tell() > (self.max_size * 1024 * 1024):
+                    log_error(f'Clipboard write request has more data than allowed by clipboard_max_size ({self.max_size}), truncating')
+                    self.max_size_exceeded = True
 
     def data_for(self, mime: str = 'text/plain', offset: int = 0, size: int = -1) -> bytes:
         start, full_size = self.mime_map[mime]
@@ -339,7 +318,7 @@ class ClipboardRequestManager:
     def parse_osc_5522(self, data: memoryview) -> None:
         import base64
 
-        from .notify import sanitize_id
+        from .notifications import sanitize_id
         idx = find_in_memoryview(data, ord(b';'))
         if idx > -1:
             metadata = str(data[:idx], "utf-8", "replace")
@@ -347,7 +326,7 @@ class ClipboardRequestManager:
         else:
             metadata = str(data, "utf-8", "replace")
             epayload = data[len(data):]
-        m: Dict[str, str] = {}
+        m: dict[str, str] = {}
         for record in metadata.split(':'):
             try:
                 k, v = record.split('=', 1)
